@@ -1,5 +1,5 @@
 /* Writing Java ResourceBundles.
-   Copyright (C) 2001-2003, 2005-2010, 2014, 2016, 2018-2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2010, 2014, 2016, 2018-2020 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -53,6 +53,7 @@
 # define S_IXUSR 00100
 #endif
 
+#include "attribute.h"
 #include "c-ctype.h"
 #include "error.h"
 #include "xerror.h"
@@ -624,7 +625,7 @@ write_java_expression (FILE *stream, const struct expression *exp, bool as_boole
               fprintf (stream, ")");
               return;
             }
-          /*FALLTHROUGH*/
+          FALLTHROUGH;
         case var:
         case mult:
         case divide:
@@ -1187,7 +1188,7 @@ msgdomain_write_java (message_list_ty *mlp, const char *canon_encoding,
 
   /* Create the Java file.  */
   register_temp_file (tmpdir, java_file_name);
-  java_file = fopen_temp (java_file_name, "w");
+  java_file = fopen_temp (java_file_name, "w", false);
   if (java_file == NULL)
     {
       error (0, errno, _("failed to create \"%s\""), java_file_name);
